"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const stringHash = require("string-hash");
const Support_1 = require("../support/Support");
const DateTime_1 = require("../DateTime");
const Combo = require("stream-json/Combo");
const Source = require("stream-json/Source");
var Tag;
(function (Tag) {
    Tag[Tag["Null"] = 0] = "Null";
    Tag[Tag["False"] = 1] = "False";
    Tag[Tag["True"] = 2] = "True";
    Tag[Tag["Integer"] = 3] = "Integer";
    Tag[Tag["Double"] = 4] = "Double";
    Tag[Tag["InternedString"] = 5] = "InternedString";
    Tag[Tag["UninternedString"] = 6] = "UninternedString";
    Tag[Tag["Object"] = 7] = "Object";
    Tag[Tag["Array"] = 8] = "Array";
    Tag[Tag["Date"] = 9] = "Date";
    Tag[Tag["Time"] = 10] = "Time";
    Tag[Tag["DateTime"] = 11] = "DateTime";
})(Tag = exports.Tag || (exports.Tag = {}));
const TAG_BITS = 4;
const TAG_MASK = (1 << TAG_BITS) - 1;
function makeValue(t, index) {
    return t | (index << TAG_BITS);
}
function getIndex(v, tag) {
    Support_1.assert(valueTag(v) === tag, "Trying to get index for value with invalid tag");
    return v >> TAG_BITS;
}
function valueTag(v) {
    return v & TAG_MASK;
}
exports.valueTag = valueTag;
class CompressedJSON {
    constructor(_makeDate, _makeTime, _makeDateTime) {
        this._makeDate = _makeDate;
        this._makeTime = _makeTime;
        this._makeDateTime = _makeDateTime;
        this._contextStack = [];
        this._strings = [];
        this._stringValues = {};
        this._objects = [];
        this._arrays = [];
        this.getStringForValue = (v) => {
            return this._strings[getIndex(v, Tag.InternedString)];
        };
        this.getObjectForValue = (v) => {
            return this._objects[getIndex(v, Tag.Object)];
        };
        this.getArrayForValue = (v) => {
            return this._arrays[getIndex(v, Tag.Array)];
        };
        this.internString = (s) => {
            if (Object.prototype.hasOwnProperty.call(this._stringValues, s)) {
                return this._stringValues[s];
            }
            const value = makeValue(Tag.InternedString, this._strings.length);
            this._strings.push(s);
            this._stringValues[s] = value;
            Support_1.assert(typeof value === "number", `Interned string value is not a number: ${value}`);
            return value;
        };
        this.internObject = (obj) => {
            const index = this._objects.length;
            this._objects.push(obj);
            return makeValue(Tag.Object, index);
        };
        this.internArray = (arr) => {
            const index = this._arrays.length;
            this._arrays.push(arr);
            return makeValue(Tag.Array, index);
        };
        this.commitValue = (value) => {
            Support_1.assert(typeof value === "number", `CompressedJSON value is not a number: ${value}`);
            if (this._ctx === undefined) {
                Support_1.assert(this._rootValue === undefined, "Committing value but nowhere to commit to - root value still there.");
                this._rootValue = value;
            }
            else if (this._ctx.currentObject !== undefined) {
                if (this._ctx.currentKey === undefined || this._ctx.currentString !== undefined) {
                    return Support_1.panic("Must have key and can't have string when committing");
                }
                this._ctx.currentObject.push(this.internString(this._ctx.currentKey), value);
                this._ctx.currentKey = undefined;
            }
            else if (this._ctx.currentArray !== undefined) {
                this._ctx.currentArray.push(value);
            }
            else {
                return Support_1.panic("Committing value but nowhere to commit to");
            }
        };
        this.finish = () => {
            const value = this._rootValue;
            if (value === undefined) {
                return Support_1.panic("Finished without root document");
            }
            Support_1.assert(this._ctx === undefined && this._contextStack.length === 0, "Finished with contexts present");
            this._rootValue = undefined;
            return value;
        };
        this.pushContext = () => {
            if (this._ctx !== undefined) {
                this._contextStack.push(this._ctx);
            }
            this._ctx = {
                currentObject: undefined,
                currentArray: undefined,
                currentKey: undefined,
                currentString: undefined,
                currentNumberIsDouble: undefined
            };
        };
        this.popContext = () => {
            Support_1.assert(this._ctx !== undefined, "Popping context when there isn't one");
            this._ctx = this._contextStack.pop();
        };
        this.handleStartObject = () => {
            this.pushContext();
            Support_1.defined(this._ctx).currentObject = [];
        };
        this.handleEndObject = () => {
            const obj = Support_1.defined(this._ctx).currentObject;
            if (obj === undefined) {
                return Support_1.panic("Object ended but not started");
            }
            this.popContext();
            this.commitValue(this.internObject(obj));
        };
        this.handleStartArray = () => {
            this.pushContext();
            Support_1.defined(this._ctx).currentArray = [];
        };
        this.handleEndArray = () => {
            const arr = Support_1.defined(this._ctx).currentArray;
            if (arr === undefined) {
                return Support_1.panic("Array ended but not started");
            }
            this.popContext();
            this.commitValue(this.internArray(arr));
        };
        this.handleStartKey = () => {
            Support_1.defined(this._ctx).currentString = "";
        };
        this.handleEndKey = () => {
            const ctx = Support_1.defined(this._ctx);
            const str = ctx.currentString;
            if (str === undefined) {
                return Support_1.panic("Key ended but no string");
            }
            ctx.currentKey = str;
            ctx.currentString = undefined;
        };
        this.handleStartString = () => {
            this.pushContext();
            Support_1.defined(this._ctx).currentString = "";
        };
        this.handleStringChunk = (s) => {
            const ctx = Support_1.defined(this._ctx);
            if (ctx.currentString === undefined) {
                return Support_1.panic("String chunk but no string");
            }
            ctx.currentString += s;
        };
        this.handleEndString = () => {
            const str = Support_1.defined(this._ctx).currentString;
            if (str === undefined) {
                return Support_1.panic("String ended but not started");
            }
            this.popContext();
            let value = undefined;
            if (str.length <= 64) {
                if (str.length > 0 && "0123456789".indexOf(str[0]) >= 0) {
                    if (this._makeDate && DateTime_1.isDate(str)) {
                        value = makeValue(Tag.Date, 0);
                    }
                    else if (this._makeTime && DateTime_1.isTime(str)) {
                        value = makeValue(Tag.Time, 0);
                    }
                    else if (this._makeDateTime && DateTime_1.isDateTime(str)) {
                        value = makeValue(Tag.DateTime, 0);
                    }
                }
                if (value === undefined) {
                    value = this.internString(str);
                }
            }
            else {
                value = makeValue(Tag.UninternedString, 0);
            }
            this.commitValue(value);
        };
        this.handleStartNumber = () => {
            this.pushContext();
            Support_1.defined(this._ctx).currentNumberIsDouble = false;
        };
        this.handleNumberChunk = (s) => {
            if (s.includes(".") || s.includes("e") || s.includes("E")) {
                Support_1.defined(this._ctx).currentNumberIsDouble = true;
            }
        };
        this.handleEndNumber = () => {
            const isDouble = Support_1.defined(this._ctx).currentNumberIsDouble;
            if (isDouble === undefined) {
                return Support_1.panic("Number ended but not started");
            }
            const numberTag = isDouble ? Tag.Double : Tag.Integer;
            this.popContext();
            this.commitValue(makeValue(numberTag, 0));
        };
        this.handleNullValue = () => {
            this.commitValue(makeValue(Tag.Null, 0));
        };
        this.handleTrueValue = () => {
            this.commitValue(makeValue(Tag.True, 0));
        };
        this.handleFalseValue = () => {
            this.commitValue(makeValue(Tag.False, 0));
        };
        this.equals = (other) => {
            return this === other;
        };
        this.hashCode = () => {
            let hashAccumulator = Support_1.hashCodeInit;
            for (const s of this._strings) {
                hashAccumulator = Support_1.addHashCode(hashAccumulator, stringHash(s));
            }
            for (const s of Object.getOwnPropertyNames(this._stringValues).sort()) {
                hashAccumulator = Support_1.addHashCode(hashAccumulator, stringHash(s));
                hashAccumulator = Support_1.addHashCode(hashAccumulator, this._stringValues[s]);
            }
            for (const o of this._objects) {
                for (const v of o) {
                    hashAccumulator = Support_1.addHashCode(hashAccumulator, v);
                }
            }
            for (const o of this._arrays) {
                for (const v of o) {
                    hashAccumulator = Support_1.addHashCode(hashAccumulator, v);
                }
            }
            return hashAccumulator;
        };
    }
    readFromStream(readStream) {
        return __awaiter(this, void 0, void 0, function* () {
            const combo = new Combo();
            const jsonSource = new Source([combo]);
            jsonSource.on("startObject", this.handleStartObject);
            jsonSource.on("endObject", this.handleEndObject);
            jsonSource.on("startArray", this.handleStartArray);
            jsonSource.on("endArray", this.handleEndArray);
            jsonSource.on("startKey", this.handleStartKey);
            jsonSource.on("endKey", this.handleEndKey);
            jsonSource.on("startString", this.handleStartString);
            jsonSource.on("stringChunk", this.handleStringChunk);
            jsonSource.on("endString", this.handleEndString);
            jsonSource.on("startNumber", this.handleStartNumber);
            jsonSource.on("numberChunk", this.handleNumberChunk);
            jsonSource.on("endNumber", this.handleEndNumber);
            jsonSource.on("nullValue", this.handleNullValue);
            jsonSource.on("trueValue", this.handleTrueValue);
            jsonSource.on("falseValue", this.handleFalseValue);
            const promise = new Promise((resolve, reject) => {
                jsonSource.on("end", () => {
                    resolve(this.finish());
                });
                combo.on("error", (err) => {
                    reject(err);
                });
            });
            readStream.setEncoding("utf8");
            readStream.pipe(jsonSource.input);
            readStream.resume();
            return promise;
        });
    }
}
exports.CompressedJSON = CompressedJSON;
