"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TypeUtils_1 = require("../TypeUtils");
const Support_1 = require("../support/Support");
const TypeAttributes_1 = require("../TypeAttributes");
const StringTypes_1 = require("../StringTypes");
const Containers_1 = require("../support/Containers");
const MIN_LENGTH_FOR_ENUM = 10;
function shouldBeEnum(enumCases) {
    Support_1.assert(enumCases.size > 0, "How did we end up with zero enum cases?");
    const someCaseIsNotNumber = Containers_1.iterableSome(enumCases.keys(), key => /^(\-|\+)?[0-9]+(\.[0-9]+)?$/.test(key) === false);
    const numValues = Containers_1.iterableReduce(enumCases.values(), 0, (a, b) => a + b);
    return numValues >= MIN_LENGTH_FOR_ENUM && enumCases.size < Math.sqrt(numValues) && someCaseIsNotNumber;
}
function expandStrings(graph, stringTypeMapping, inference, debugPrintReconstitution) {
    function replaceString(group, builder, forwardingRef) {
        Support_1.assert(group.size === 1);
        const t = Support_1.defined(Containers_1.iterableFirst(group));
        const stringTypes = TypeUtils_1.stringTypesForType(t);
        const attributes = Containers_1.mapFilter(t.getAttributes(), a => a !== stringTypes);
        const mappedStringTypes = stringTypes.applyStringTypeMapping(stringTypeMapping);
        if (!mappedStringTypes.isRestricted) {
            return builder.getStringType(attributes, StringTypes_1.StringTypes.unrestricted, forwardingRef);
        }
        const types = [];
        const cases = Support_1.defined(mappedStringTypes.cases);
        if (cases.size > 0) {
            if (inference === "all" || (inference === "infer" && shouldBeEnum(cases))) {
                types.push(builder.getEnumType(TypeAttributes_1.emptyTypeAttributes, new Set(cases.keys())));
            }
            else {
                return builder.getStringType(attributes, StringTypes_1.StringTypes.unrestricted, forwardingRef);
            }
        }
        if (mappedStringTypes.allowDate) {
            types.push(builder.getPrimitiveType("date"));
        }
        if (mappedStringTypes.allowTime) {
            types.push(builder.getPrimitiveType("time"));
        }
        if (mappedStringTypes.allowDateTime) {
            types.push(builder.getPrimitiveType("date-time"));
        }
        Support_1.assert(types.length > 0, "We got an empty string type");
        return builder.getUnionType(attributes, new Set(types), forwardingRef);
    }
    const allStrings = Array.from(graph.allTypesUnordered())
        .filter(t => t.kind === "string" && TypeUtils_1.stringTypesForType(t).isRestricted)
        .map(t => [t]);
    return graph.rewrite("expand strings", stringTypeMapping, false, allStrings, debugPrintReconstitution, replaceString);
}
exports.expandStrings = expandStrings;
